package util

import (
	"fmt"
	"github.com/antonholmquist/jason"
	"io/ioutil"
	//	"log"
	//	"os"
	//	"bytes"
	"strings"
)

type Config struct {
	// The JSON object of what was read
	LoadedJson      jason.Object
	Options         jason.Object
	FeatureTPHome   bool
	FeatureTPVisit  bool
	FeatureDayNight bool
	Users           []*User
}

var c *Config

func LoadConfig(mm *MessageManager) {
	c = new(Config)
	config_str, err := ioutil.ReadFile("mc_man.config")
	if err == nil {
		j, _ := jason.NewObjectFromBytes(config_str)
		o, _ := j.GetObjectArray("options")

		// Add the "Stop" listener
		fmt.Println("Activating 'stop' listener")
		AddListener(func(i *Message) bool {
			//listeners = append(mm.listeners, func(i *Message) bool {
			if i.User.IsOp && i.Text == "!stop\n" {
				mm.Output("stop")
				return true
			}
			return false
		})

		for _, option := range o {
			opt_name, _ := option.GetString("name")
			opt_enabled, _ := option.GetBoolean("enabled")
			if opt_name == "home" {
				c.FeatureTPHome = opt_enabled
				if opt_enabled {
					fmt.Println("Activating 'home' listeners")
					// Add !set home listener
					AddListener(func(i *Message) bool {
						if i.User.Name != "" && i.Text == "!set home\n" {
							AddTempListener(func(inp *Message) bool {
								listen_for := "Teleported " + i.User.Name + " to "
								if inp.User.Name == "" && strings.Contains(inp.Text, listen_for) {
									// Found the text
									r := strings.Split(inp.Text, listen_for)
									if len(r) > 0 {
										p_str := r[1]
										p_str = strings.Replace(p_str, ",", "", -1)
										SetHome(i.User.Name, p_str)
										mm.Output("tellraw " + i.User.Name + " Set your home to " + p_str)
										return true
									}
								}
								return false
							})
							mm.Output("tp " + i.User.Name + " ~ ~ ~")
							return true
						}
						return false
					})
					// Add !home listener
					AddListener(func(i *Message) bool {
						if i.User.Name != "" && i.Text == "!home\n" {
							home_str, found := GetHome(i.User.Name)
							if found {
								mm.Output("tp " + i.User.Name + " " + home_str)
							} else {
								mm.Output("tellraw " + i.User.Name + " I don't know where your home is. Set it to your current position by typing '!set home'")
							}
						}
						return false
					})
				}
			} else if opt_name == "visit" {
				c.FeatureTPVisit = opt_enabled
				if opt_enabled {
					fmt.Println("Activating 'visit' listeners")
					// Add !set porch listener
					AddListener(func(i *Message) bool {
						if i.User.Name != "" && i.Text == "!set porch\n" {
							AddTempListener(func(inp *Message) bool {
								listen_for := "Teleported " + i.User.Name + " to "
								if inp.User.Name == "" && strings.Contains(inp.Text, listen_for) {
									// Found the text
									r := strings.Split(inp.Text, listen_for)
									if len(r) > 0 {
										p_str := r[1]
										p_str = strings.Replace(p_str, ",", "", -1)
										SetPorch(i.User.Name, p_str)
										mm.Output("tellraw " + i.User.Name + " Set your porch to " + p_str)
										return true
									}
								}
								return false
							})
							mm.Output("tp " + i.User.Name + " ~ ~ ~")
							return true
						}
						return false
					})
					// Add !visit listener
					AddListener(func(i *Message) bool {
						if i.User.Name != "" && strings.HasPrefix(i.Text, "!visit ") {
							// Find the user we're trying to visit
							r := strings.Split(strings.Replace(i.Text, "\n", "", -1), "!visit ")
							if len(r) > 0 {
								username := r[1]
								porch_str, found := GetPorch(username)
								if found {
									mm.Output("tp " + i.User.Name + " " + porch_str)
								} else {
									mm.Output("tellraw " + i.User.Name + " I don't know where " + username + "s porch is. They can set it to their current position by typing '!set porch'")
								}
							}
						}
						return false
					})
				}
			} else if opt_name == "daynight" {
				c.FeatureDayNight = opt_enabled
				if opt_enabled {
					fmt.Println("Activating 'daynight' listeners")
					// Add !switch day listener
					AddListener(func(i *Message) bool {
						if i.User.Name != "" && i.Text == "!switch day\n" {
							// TODO: Start vote
							mm.Output("time set day")
							return true
						}
						return false
					})
					// Add !switch night listener
					AddListener(func(i *Message) bool {
						if i.User.Name != "" && i.Text == "!switch night\n" {
							// TODO: Start vote
							mm.Output("time set night")
							return true
						}
						return false
					})
				}
			}
			// Add !help listener
			AddListener(func(i *Message) bool {
				if i.User.Name != "" && i.Text == "!help\n" {
					hlp_txt := " {text:\"-=( mc_man Manager Help )=-\",color:blue}"
					mm.Output("tellraw " + i.User.Name + hlp_txt)
					hlp_txt = " {text:\"!set home -- Set your 'home' to your current position.\"}"
					mm.Output("tellraw " + i.User.Name + hlp_txt)
					hlp_txt = " {text:\"!home -- Request a teleport to your 'home' position.\"}"
					mm.Output("tellraw " + i.User.Name + hlp_txt)
					hlp_txt = " {text:\"!set porch -- Set your 'porch' to your current position.\"}"
					mm.Output("tellraw " + i.User.Name + hlp_txt)
					hlp_txt = " {text:\"!visit <username> -- Request a teleport to <username>'s 'porch' position.\"}"
					mm.Output("tellraw " + i.User.Name + hlp_txt)
					hlp_txt = " {text:\"!switch day -- Ask the server to switch the time to 'day'.\"}"
					mm.Output("tellraw " + i.User.Name + hlp_txt)
					hlp_txt = " {text:\"!switch night -- Ask the server to switch the time to 'night'.\"}"
					mm.Output("tellraw " + i.User.Name + hlp_txt)
					hlp_txt = " {text:\"-=========================-\",color:blue}"
					mm.Output("tellraw " + i.User.Name + hlp_txt)
					return true
				}
				return false
			})
		}

		c.Users = make([]*User, 0, 10)
		u, _ := j.GetObjectArray("users")
		for _, user := range u {
			user_name, err := user.GetString("name")
			if err == nil && user_name != "" {
				user_home, _ := user.GetString("home")
				user_porch, _ := user.GetString("porch")
				us := NewUser(user_name)
				// TODO: Check if this user is an op on the server
				us.Home = user_home
				us.Porch = user_porch
				c.Users = append(c.Users, us)
			}
		}
		fmt.Printf("Loaded %d Users\n", len(c.Users))
	}
}

func AddUser(username string) {
	us := NewUser(username)
	c.Users = append(c.Users, us)
	WriteConfig()
}

func WriteConfig() {
	// Generate the JSON string for the config file
	d := "{\"options\":["
	// Output options array
	d = d + "{\"name\":\"home\",\"enabled\":"
	if c.FeatureTPHome {
		d = d + "true"
	} else {
		d = d + "false"
	}
	d = d + "},{\"name\":\"visit\",\"enabled\":"
	if c.FeatureTPVisit {
		d = d + "true"
	} else {
		d = d + "false"
	}
	d = d + "},{\"name\":\"daynight\",\"enabled\":"
	if c.FeatureDayNight {
		d = d + "true"
	} else {
		d = d + "false"
	}
	d = d + "}],\"users\":["
	// Output users array
	for _, u := range c.Users {
		d = d + u.ToJSONString()
	}
	/*
		num_users := len(c.Users)
		fmt.Printf("Number of Users: %d", num_users)
		for i := 0; i < num_users; i++ {
			user := c.Users[i]
			//	for _, user := range c.Users {
			fmt.Println(d)
			num_users--
			d = d + user.ToJSONString()
			if user.Name != "" {
				if num_users > 0 {
					d = d + ","
				}
			}
		}
	*/
	d = d + "]}"
	do := []byte(d)
	ioutil.WriteFile("mc_man.config", do, 0664)
}

func SetHome(user string, loc string) {
	u, idx := FindUser(user)
	if idx == -1 {
		u = NewUser(user)
		c.Users = append(c.Users, u)
		idx = len(c.Users) - 1
	}
	u.Home = strings.Replace(loc, "\n", "", -1)
	// Replace the user in the Users array
	c.Users[idx] = u
	WriteConfig()
}

func GetHome(user string) (string, bool) {
	u, idx := FindUser(user)
	if idx == -1 || u.Home == "" {
		return "", false
	}
	return u.Home, true
}

func SetPorch(user string, loc string) {
	u, idx := FindUser(user)
	if idx == -1 {
		u = NewUser(user)
		c.Users = append(c.Users, u)
		idx = len(c.Users) - 1
	}
	u.Porch = strings.Replace(loc, "\n", "", -1)
	c.Users[idx] = u
	WriteConfig()
}

func GetPorch(user string) (string, bool) {
	u, idx := FindUser(user)
	if idx == -1 || u.Porch == "" {
		return "", false
	}
	return u.Porch, true
}

func FindUser(name string) (*User, int) {
	for i, user := range c.Users {
		if user.Name == name {
			return user, i
		}
	}
	return nil, -1
}
